<?php

namespace Drupal\schema_ui\Controller;

use Drupal\Core\Config\Entity\ConfigEntityTypeInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\Sql\SqlEntityStorageInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Link;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Drupal\field\Entity\FieldConfig;

/**
 * Provides route controllers for Schema UI.
 */
class EntitySchemaController extends ControllerBase {

  use StringTranslationTrait;

  public function title(RouteMatchInterface $routeMatch) {
    $entity_type_id = $routeMatch->getParameter('entity_type_id');
    $definition = $this->entityTypeManager()->getDefinition($entity_type_id);

    $label = $definition->getLabel();
    $config_entity_type = $routeMatch->getParameter($entity_type_id);
    if ($config_entity_type instanceof EntityInterface) {
      $label = $config_entity_type->label();
    }

    return $this->t('%label schema', [
      '%label' => $label,
    ]);
  }

  public function fieldTitle(RouteMatchInterface $routeMatch) {
    $field_config_id = $routeMatch->getParameter('field_config');
    $field_config_storage = $this->entityTypeManager()->getStorage('field_config');
    /** @var FieldConfig $field */
    $field = $field_config_storage->load($field_config_id);

    return $this->t('%field field schema', [
      '%field' => $field->label(),
    ]);
  }

  public function field(RouteMatchInterface $routeMatch) {
    $field_config_id = $routeMatch->getParameter('field_config');
    $field_config_storage = $this->entityTypeManager()->getStorage('field_config');
    /** @var FieldConfig $field */
    $field = $field_config_storage->load($field_config_id);
    $rows = [];
    $this->buildFieldTableRows($field->get('field_name'), $field, $rows);

    return $this->buildTable($rows);
  }

  public function content(RouteMatchInterface $routeMatch) {
    $entity_type_id = $routeMatch->getParameter('entity_type_id');
    $bundle = $bundle_entity_type = $entity_type_id;

    $config_entity_type = $routeMatch->getParameter($entity_type_id);
    if ($config_entity_type instanceof EntityInterface) {
      $bundle = $config_entity_type->getOriginalId();
      $config_entity_type_definition = $this->entityTypeManager()->getDefinition($entity_type_id);
      $bundle_entity_type = $config_entity_type_definition->getBundleOf();
    }

    // list all fields of entity type
    $fields = \Drupal::service('entity_field.manager')->getFieldDefinitions($bundle_entity_type, $bundle);

    $rows = [];
    foreach ($fields as $field_name => $field) {

      $this->buildFieldTableRows($field_name, $field, $rows);

      if ($field instanceof FieldConfig) {
        $field_route = "entity.field_config.{$bundle_entity_type}_field_edit_form";
        $rows[$field_name][0]['data']['#value'] = Link::createFromRoute($field_name, $field_route, [
          'field_config' => $field->id(),
          $entity_type_id => $bundle,
        ])->toString();
      }
    }

    return $this->buildTable($rows);

  }

  public function table(RouteMatchInterface $routeMatch) {
    $entity_type_id = $routeMatch->getParameter('entity_type_id');
    $bundle = $bundle_entity_type = $entity_type_id;

    $config_entity_type = $routeMatch->getParameter($entity_type_id);
    if ($config_entity_type instanceof EntityInterface) {
      $bundle = $config_entity_type->getOriginalId();
      $config_entity_type_definition = $this->entityTypeManager()->getDefinition($entity_type_id);
      $bundle_entity_type = $config_entity_type_definition->getBundleOf();
    }

    $build = [];
    $build['table'] = [
      '#type' => 'table',
      '#header' => [
        'table' => $this->t('Table'),
        'fields' => $this->t('Fields')
      ],
      '#rows' => [],
      '#empty' => t('No content has been found.'),
      '#attached' => [
        'library' => [
          'schema_ui/table_styling',
        ],
      ],
      '#attributes' => [
        'class' => [
          'schema-ui-table',
        ],
      ],
    ];

    $entity_storage = $this->entityTypeManager()->getStorage($bundle_entity_type);
    if ($entity_storage instanceof SqlEntityStorageInterface) {
      $table_mapping = $entity_storage->getTableMapping();
      foreach ($table_mapping->getTableNames() as $delta => $table_name) {

        $build['table']['#rows'][$delta]['table'] = $table_name;
        $build['table']['#rows'][$delta]['fields'] = implode(', ', $table_mapping->getAllColumns($table_name));
      }
    }

    return $build;
  }

  protected function buildTable($rows) {
    $header = [
      'id' => t('Key'),
      'type' => t('Type'),
      'cardinality' => t('Cardinality'),
      'title' => t('Title'),
    ];
    $build['table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => t('No content has been found.'),
      '#attached' => [
        'library' => [
          'schema_ui/table_styling',
        ],
      ],
      '#attributes' => [
        'class' => [
          'schema-ui-table',
        ],
      ],
    ];
    return $build;
  }

  protected function buildFieldTableRows($field_name, $field, &$rows) {
    $cartinality = NULL;
    $columns = [];

    if ($field instanceof FieldStorageDefinitionInterface) {
      $columns = $field->getColumns();
      $cartinality = $field->getCardinality();
    }
    if ($field instanceof FieldConfig) {
      $columns = $field->getFieldStorageDefinition()->getColumns();
      $cartinality = $field->getFieldStorageDefinition()->getCardinality();
    }

    if ($cartinality === BaseFieldDefinition::CARDINALITY_UNLIMITED) {
      $cartinality = 'Unlimited';
    }

    if (is_int($cartinality)) {
      $cartinality = 'Limited ('.$cartinality.')';
    }

    $rows[$field_name] = [
      [
        'data' => [
          '#type' => 'html_tag',
          '#tag' => 'code',
          '#value' => $field_name,
          '#attributes' => [
            'class' => [
              'field-definition',
            ],
          ],
        ]
      ],
      [
        'data' => [
          '#type' => 'html_tag',
          '#tag' => 'code',
          '#value' => $field->getType(),
          '#attributes' => [
            'class' => [
              'field-definition',
            ],
          ],
        ],
      ],
      $cartinality,
      $field->getLabel()
    ];

    foreach ($columns as $key => $item) {
      $rows[$field_name . $key] = [
        [
          'data' => [
            '#markup' => sprintf('└─ <code>%s</code>', $key),
          ]
        ],
        $item['type'],
        $item['length'] ?? $item['size'] ?? '',

        [
          'data' => [
            '#type' => 'html_tag',
            '#tag' => 'small',
            '#value' => $item['description'] ?? '',
            '#attributes' => [
              'class' => [
                '',
              ],
            ],
          ]
        ],
      ];
      if (isset($item['length'])) {
        $rows[$field_name . $key][2] = $this->formatPlural(
          $item['length'],
          '@count character',
          '@count characters'
        );
      }

    }

  }

}
