<?php

namespace Drupal\schema_ui\Plugin\Derivative;

use Drupal\Core\Config\Entity\ConfigEntityTypeInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\ContentEntityTypeInterface;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Component\Plugin\Derivative\DeriverBase;
use Drupal\Core\Plugin\Discovery\ContainerDeriverInterface;
use Drupal\Core\Routing\RouteProviderInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides local task definitions for all entity bundles.
 */
class SchemaUiLocalTask extends DeriverBase implements ContainerDeriverInterface {
  use StringTranslationTrait;

  /**
   * The route provider.
   *
   * @var \Drupal\Core\Routing\RouteProviderInterface
   */
  protected $routeProvider;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity display repository.
   *
   * @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface
   */
  protected $entityDisplayRepository;

  /**
   * Creates a FieldUiLocalTask object.
   *
   * @param \Drupal\Core\Routing\RouteProviderInterface $route_provider
   *   The route provider.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $string_translation
   *   The translation manager.
   * @param \Drupal\Core\Entity\EntityDisplayRepositoryInterface $entity_display_repository
   *   The entity display repository.
   */
  public function __construct(RouteProviderInterface $route_provider, EntityTypeManagerInterface $entity_type_manager, TranslationInterface $string_translation, EntityDisplayRepositoryInterface $entity_display_repository) {
    $this->routeProvider = $route_provider;
    $this->entityTypeManager = $entity_type_manager;
    $this->stringTranslation = $string_translation;
    $this->entityDisplayRepository = $entity_display_repository;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, $base_plugin_id) {
    return new static(
      $container->get('router.route_provider'),
      $container->get('entity_type.manager'),
      $container->get('string_translation'),
      $container->get('entity_display.repository')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDerivativeDefinitions($base_plugin_definition) {
    $this->derivatives = [];

    $definitions = $this->entityTypeManager->getDefinitions();

    foreach ($definitions as $entity_type_id => $entity_type) {
      if ($entity_type instanceof ConfigEntityTypeInterface) {
        $local_task_key = sprintf('schema_ui.schema:%s', $entity_type_id);

        if (!isset($definitions[$entity_type->getBundleOf()])) {
          continue;
        }

        $bundle_entity_type_id = $entity_type->getBundleOf();
        $bundle_entity_type = $definitions[$bundle_entity_type_id];

        if ($bundle_entity_type instanceof ContentEntityTypeInterface) {
          // Entity schema tab.
          $this->derivatives[$local_task_key] = [
            'route_name' => "entity.$entity_type_id.schema",
            'weight' => 100,
            'title' => $this->t('Schema'),
            'base_route' => $bundle_entity_type->get('field_ui_base_route'),
          ];

          $this->derivatives[$local_task_key.'_schema'] = [
            'route_name' => "entity.$entity_type_id.schema",
            'weight' => 100,
            'title' => $this->t('Fields'),
            'base_route' => "entity.$entity_type_id.schema",
            'parent_id' => $bundle_entity_type->get('field_ui_base_route'),
          ];

          $this->derivatives[$local_task_key.'_tables'] = [
            'route_name' => "entity.$entity_type_id.schema_tables",
            'weight' => 100,
            'title' => $this->t('Tables'),
            'base_route' => "entity.$entity_type_id.schema",
            'parent_id' => $bundle_entity_type->get('field_ui_base_route'),
          ];

          // Field schema tab.
          $this->derivatives["field_storage_$entity_type_id"] = [
            'route_name' => "entity.field_config.{$bundle_entity_type_id}_schema",
            'title' => $this->t('Schema'),
            'base_route' => "entity.field_config.{$bundle_entity_type_id}_field_edit_form",
          ];
        }
      }

      if ($entity_type instanceof ContentEntityTypeInterface && $entity_type->get('field_ui_base_route')) {
        if ($entity_type->getBundleEntityType()) {
          // Already added to the bundle entity type above.
          continue;
        }
        $this->derivatives[$local_task_key] = [
          'route_name' => "entity.$entity_type_id.schema",
          'weight' => 100,
          'title' => $this->t('Schema'),
          'base_route' => $entity_type->get('field_ui_base_route'),
        ];

        // Field schema tab.
        $this->derivatives["field_storage_$entity_type_id"] = [
          'route_name' => "entity.field_config.{$entity_type_id}_schema",
          'title' => $this->t('Schema'),
          'base_route' => "entity.field_config.{$entity_type_id}_field_edit_form",
        ];
      }
    }

    foreach ($this->derivatives as &$entry) {
      $entry += $base_plugin_definition;
    }

    return $this->derivatives;
  }

}
