<?php

namespace Drupal\schema_ui\Routing;

use Drupal\Core\Config\Entity\ConfigEntityInterface;
use Drupal\Core\Config\Entity\ConfigEntityTypeInterface;
use Drupal\Core\Entity\ContentEntityTypeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Routing\RouteSubscriberBase;
use Drupal\Core\Routing\RoutingEvents;
use Symfony\Component\Routing\Route;
use Symfony\Component\Routing\RouteCollection;

/**
 * Subscriber for Field UI routes.
 */
class RouteSubscriber extends RouteSubscriberBase {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a RouteSubscriber object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  protected function alterRoutes(RouteCollection $collection) {
    $definitions = $this->entityTypeManager->getDefinitions();

    foreach ($definitions as $entity_type_id => $entity_type) {
      if ($entity_type instanceof ConfigEntityTypeInterface) {
        $bundle_entity_type = $entity_type->getBundleOf();
        if (!$bundle_entity_type || !isset($definitions[$bundle_entity_type])) {
          continue;
        }

        $bundle_entity = $definitions[$bundle_entity_type];
        $route_name = $bundle_entity->get('field_ui_base_route');
        // Try to get the route from the current collection.
        if (!$entity_route = $collection->get($route_name)) {
          continue;
        }

        $path = $entity_route->getPath();

        $options = $entity_route->getOptions();
        $options['parameters'][$entity_type_id] = [
          'type' => 'entity:' . $entity_type_id,
        ];

        $options['_admin_route'] = TRUE;

        $defaults = [
          'entity_type_id' => $entity_type_id,
        ];

        $route = new Route(
          "$path/schema",
          [
            '_controller' => '\Drupal\schema_ui\Controller\EntitySchemaController::content',
            '_title_callback' => '\Drupal\schema_ui\Controller\EntitySchemaController::title',
          ] + $defaults,
          ['_permission' => $entity_type->get('admin_permission')],
          $options
        );
        $collection->add("entity.{$entity_type_id}.schema", $route);

        $route = new Route(
          "$path/schema/tables",
          [
            '_controller' => '\Drupal\schema_ui\Controller\EntitySchemaController::table',
            '_title_callback' => '\Drupal\schema_ui\Controller\EntitySchemaController::title',
          ] + $defaults,
          ['_permission' => $entity_type->get('admin_permission')],
          $options
        );
        $collection->add("entity.{$entity_type_id}.schema_tables", $route);

        $route = new Route(
          "$path/fields/{field_config}/schema",
          [
            '_controller' => '\Drupal\schema_ui\Controller\EntitySchemaController::field',
            '_title_callback' => '\Drupal\schema_ui\Controller\EntitySchemaController::fieldTitle',
          ] + $defaults,
          ['_permission' => 'administer ' . $bundle_entity_type . ' fields'],
          $options
        );
        $collection->add("entity.field_config.{$bundle_entity_type}_schema", $route);
      }

      if ($entity_type instanceof ContentEntityTypeInterface && $entity_type->get('field_ui_base_route')) {
        $field_ui_base_route = $entity_type->get('field_ui_base_route');
        if ($entity_type->getBundleEntityType()) {
          // Already added to the bundle entity type above.
          continue;
        }

        $route_name = $entity_type->get('field_ui_base_route');
        // Try to get the route from the current collection.
        if (!$entity_route = $collection->get($route_name)) {
          continue;
        }

        $path = $entity_route->getPath();

        $options = $entity_route->getOptions();
        $options['parameters'][$entity_type_id] = [
          'type' => 'entity:' . $entity_type_id,
        ];

        $options['_admin_route'] = TRUE;

        $defaults = [
          'entity_type_id' => $entity_type_id,
        ];

        $route = new Route(
          "$path/schema",
          [
            '_controller' => '\Drupal\schema_ui\Controller\EntitySchemaController::content',
            '_title_callback' => '\Drupal\schema_ui\Controller\EntitySchemaController::title',
          ] + $defaults,
          ['_permission' => $entity_type->get('admin_permission')],
          $options
        );
        $collection->add("entity.{$entity_type_id}.schema", $route);

        $route = new Route(
          "$path/schema/tables",
          [
            '_controller' => '\Drupal\schema_ui\Controller\EntitySchemaController::table',
            '_title_callback' => '\Drupal\schema_ui\Controller\EntitySchemaController::title',
          ] + $defaults,
          ['_permission' => $entity_type->get('admin_permission')],
          $options
        );
        $collection->add("entity.{$entity_type_id}.schema_tables", $route);

        $route = new Route(
          "$path/fields/{field_config}/schema",
          [
            '_controller' => '\Drupal\schema_ui\Controller\EntitySchemaController::field',
            '_title_callback' => '\Drupal\schema_ui\Controller\EntitySchemaController::fieldTitle',
          ] + $defaults,
          ['_permission' => 'administer ' . $entity_type_id . ' fields'],
          $options
        );
        $collection->add("entity.field_config.{$entity_type_id}_schema", $route);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events = parent::getSubscribedEvents();
    $events[RoutingEvents::ALTER] = ['onAlterRoutes', -100];
    return $events;
  }

}
